<?php

declare(strict_types=1);

namespace Drupal\field_ipaddress_pgsql\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation of the 'IP Address PostgreSQL cidr' field type.
 *
 * @FieldType(
 *   id = "ipaddress_cidr",
 *   label = @Translation("IP Address PostgreSQL cidr"),
 *   description = @Translation("Stores an IP address using the PostgreSQL cidr data type."),
 *   default_widget = "ipaddress_cidr",
 *   default_formatter = "ipaddress_cidr",
 *   constraints = {"CidrFormat" = {}}
 * )
 */
final class IpAddressCidrField extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings(): array {
    return [] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty(): bool {
    return match ($this->get('value')->getValue()) {
      NULL, '' => TRUE,
      default => FALSE,
    };
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition): array {
    $properties = [];
    $properties['value'] = DataDefinition::create('any')
      ->setLabel(t('Value'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition): array {
    $columns = [
      'value' => [
        'type' => 'text',
        'pgsql_type' => 'cidr',
        'not null' => TRUE,
        'description' => 'The IP address stored as the PostgreSQL cidr type.',
      ],
    ];

    // Adding the GIST index failed, the hack that is suggested here
    // https://www.drupal.org/node/3264989 didn't work.
    // Also, there is an open issue:
    // https://www.drupal.org/project/drupal/issues/3397622
    $indexes = [
      'value' => [
        'value',
      ],
    ];

    $schema = [
      'columns' => $columns,
      'indexes' => $indexes,
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition): array {
    $values['value'] = '192.168.100.128/25';
    return $values;
  }

}
