<?php

declare(strict_types=1);

namespace Drupal\field_ipaddress_pgsql\Plugin\Validation\Constraint;

use Drupal\Core\Database\Connection;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Validates the cidr format validation constraint.
 */
final class CidrFormatConstraintValidator extends ConstraintValidator implements ContainerInjectionInterface {

  /**
   * Constructs the object.
   */
  public function __construct(
    private readonly Connection $connection,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('database'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function validate(mixed $value, Constraint $constraint): void {
    $value = trim($value->value);

    try {
      $this->connection->query('SELECT cidr(:ipAddressString)', [':ipAddressString' => $value]);
    }
    catch (\Exception $e) {
      $this->context->addViolation($constraint->message);
    }

  }

}
